<?php

namespace Drupal\danse_content\Plugin\Danse;

use Drupal;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\ContentEntityDeleteForm;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityMalformedException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\PayloadInterface;
use Drupal\danse\PluginBase;
use Drupal\danse_content\Payload;
use Drupal\danse_content\SubscriptionOperation;
use Drupal\danse_content\Topic\Comment;
use Drupal\danse_content\Topic\Create;
use Drupal\danse_content\Topic\Delete;
use Drupal\danse_content\Topic\Publish;
use Drupal\danse_content\Topic\TopicInterface;
use Drupal\danse_content\Topic\Unpublish;
use Drupal\danse_content\Topic\Update;
use InvalidArgumentException;

/**
 * Plugin implementation of DANSE.
 *
 * @Danse(
 *   id = "content",
 *   label = @Translation("Content"),
 *   description = @Translation("Manages all content entity types and their
 *   bundles.")
 * )
 */
class Content extends PluginBase {

  use ContentSettingsTrait;

  public const SETTING_CREATE_EVENT = 'create_event';

  /**
   * @return \Drupal\danse_content\Topic\TopicInterface[]
   */
  public static function topics(): array {
    static $topics;
    if ($topics === NULL) {
      $topics = [];
      if ($container = Drupal::getContainer()) {
        $topics = [
          Create::create($container),
          Update::create($container),
          Delete::create($container),
          Publish::create($container),
          Unpublish::create($container),
          Comment::create($container),
        ];
      }
    }
    return $topics;
  }

  /**
   * @param string $topic_id
   *
   * @return \Drupal\danse_content\Topic\TopicInterface
   */
  public static function topic(string $topic_id): TopicInterface {
    foreach (self::topics() as $topic) {
      if ($topic->id() === $topic_id) {
        return $topic;
      }
    }
    throw new InvalidArgumentException('Invalid topic id');
  }

  /**
   * {@inheritdoc}
   */
  public function assertPayload(PayloadInterface $payload): bool {
    return $payload instanceof Payload;
  }

  /**
   * {@inheritdoc}
   */
  public function assertSubscriptionKey() {
    if (count($this->subscriptionKeyParts) === 4) {
      [$entityType, $bundle, $subscriptionMode, $topicId] = $this->subscriptionKeyParts;
      if ($subscriptionMode === SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY_TYPE) {
        $subscriptionType = 'allow_subscription';
        $entity = TRUE;
      }
      else {
        if ($subscriptionMode === SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY) {
          $subscriptionType = 'allow_entity_subscription';
        }
        else if ($subscriptionMode === SubscriptionOperation::SUBSCRIPTION_MODE_RELATED_ENTITY) {
          $subscriptionType = 'allow_related_entity_subscription';
        }
        else {
          return FALSE;
        }
        try {
          $entity = $this->entityTypeManager->getStorage($entityType)->load($bundle);
        }
        catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
          return FALSE;
        }
        if ($entity === NULL) {
          return FALSE;
        }
        $bundle = $entity->bundle();
      }

      $topic = self::topic($topicId);
      $config_id = $this->configId($entityType, $bundle);
      $config = $this->configFactory->get($config_id);
      if ($roles = $config->get($topic->getSettingKey($subscriptionType))) {
        $userRoles = $this->currentUser->getRoles();
        foreach ($roles as $role) {
          if (in_array($role, $userRoles, TRUE)) {
            return $entity;
          }
        }
      }
    }
    return FALSE;
  }

  /**
   * @return array
   */
  public function enabledSubscriptionBundles(): array {
    $result = [];
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type => $definition) {
      $result[$entity_type] = [];
      foreach ($this->bundleInfo->getBundleInfo($entity_type) as $bundle => $bundleDef) {
        $config_id = $this->configId($definition->id(), $bundle);
        $config = $this->configFactory->get($config_id);
        foreach (['allow_subscription', 'allow_entity_subscription', 'allow_related_entity_subscription'] as $subscriptionType) {
          foreach (self::topics() as $topic) {
            if ($topic->assert($definition->id()) && $roles = $config->get($topic->getSettingKey($subscriptionType))) {
              $result[$entity_type][] = $bundle;
              break 2;
            }
          }
        }
      }
    }
    return $result;
  }

  /**
   * @return array
   */
  public function enabledSubscriptionEntityTypes(): array {
    $result = [];
    foreach ($this->enabledSubscriptionBundles() as $entity_type => $enabledSubscriptionBundle) {
      if (!empty($enabledSubscriptionBundle)) {
        $result[] = $entity_type;
      }
    }
    return $result;
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return \Drupal\danse_content\SubscriptionOperation[]
   */
  public function subscriptionOperations(ContentEntityInterface $entity): array {
    $operations = [];
    $config_id = $this->configId($entity->getEntityTypeId(), $entity->bundle());
    $config = $this->configFactory->get($config_id);
    $userRoles = $this->currentUser->getRoles();
    $userId = $this->currentUser->id();
    $subscriptionTypes = ['allow_subscription' => SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY_TYPE];
    if ($entity->id()) {
      $subscriptionTypes['allow_entity_subscription'] = SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY;
      $subscriptionTypes['allow_related_entity_subscription'] = SubscriptionOperation::SUBSCRIPTION_MODE_RELATED_ENTITY;
    }
    foreach ($subscriptionTypes as $subscriptionType => $subscriptionMode) {
      foreach (self::topics() as $topic) {
        if ($topic->assert($entity->getEntityTypeId()) && !empty($config->get($topic->getSettingKey($subscriptionType)))) {
          foreach ($config->get($topic->getSettingKey($subscriptionType)) as $rid) {
            if (in_array($rid, $userRoles, TRUE)) {
              if ($subscriptionMode === SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY_TYPE) {
                $key = $this->subscriptionKey($entity->getEntityTypeId(), $entity->bundle(), $subscriptionMode, $topic->id());
              }
              else {
                $key = $this->subscriptionKey($entity->getEntityTypeId(), $entity->id(), $subscriptionMode, $topic->id());
              }
              $operation = new SubscriptionOperation($entity, $key, $topic, $subscriptionMode);
              if ($this->userData->get('danse', $userId, $key)) {
                $operation->setUnsubscribeMode();
              }
              $operations[] = $operation;
              // Do not try for other roles as we already found one.
              break;
            }
          }
        }
      }
    }
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedSubscriptions($roles): array {
    $subscriptions = [];
    foreach ($this->entityTypeManager->getDefinitions() as $definition) {
      if ($definition instanceof ContentEntityTypeInterface) {
        $bundles = $this->bundleInfo->getBundleInfo($definition->id());
        if (empty($bundles)) {
          $bundles = [$definition->id() => []];
        }
        $numberOfBundles = count($bundles);
        foreach ($bundles as $bundle => $bundleInfo) {
          $config_id = $this->configId($definition->id(), $bundle);
          $config = $this->configFactory->get($config_id);
          foreach (self::topics() as $topic) {
            if ($topic->assert($definition->id())) {
              $relevantRoles = array_intersect(
                $config->get($topic->getSettingKey('allow_subscription')) ?? [],
                $roles
              );
              if (!empty($relevantRoles)) {
                $subscriptions[$this->subscriptionKey($definition->id(), $bundle, SubscriptionOperation::SUBSCRIPTION_MODE_ENTITY_TYPE, $topic->id())] =
                  empty($bundleInfo) || $numberOfBundles === 1 ?
                    $this->t('@topic @entity_type', [
                      '@entity_type' => strtolower($definition->getLabel()),
                      '@topic' => $topic->id(),
                    ]) :
                    $this->t('@topic @entity_type of type @bundle', [
                      '@bundle' => strtolower($bundleInfo['label']),
                      '@entity_type' => strtolower($definition->getLabel()),
                      '@topic' => $topic->id(),
                    ]);
              }
            }
          }
        }
      }
    }
    return $subscriptions;
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return \Drupal\danse_content\Topic\TopicInterface[]
   */
  public function topicsForEntity(ContentEntityInterface $entity, bool $force_create_topic = FALSE): array {
    $topics = ($force_create_topic || $entity->isNew()) ?
      [self::topic(TopicInterface::CREATE)] :
      [self::topic(TopicInterface::UPDATE)];
    if (!$force_create_topic && $this->isPublished($entity)) {
      $topics[] = self::topic(TopicInterface::UNPUBLISH);
    }
    else {
      $topics[] = self::topic(TopicInterface::PUBLISH);
    }
    return $topics;
  }

  /**
   * @return \Drupal\danse_content\Topic\TopicInterface[]
   */
  public function topicsForEntityDeletion(): array {
    return [
      self::topic(TopicInterface::DELETE),
      self::topic(TopicInterface::UNPUBLISH),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(&$form, FormStateInterface $form_state) {
    $info = $form_state->getBuildInfo();
    if (!empty($info['callback_object']) && $info['callback_object'] instanceof ContentEntityFormInterface) {
      $callback = $info['callback_object'];
      /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
      $entity = $callback->getEntity();
      if ($callback instanceof ContentEntityDeleteForm) {
        $topics = $this->topicsForEntityDeletion();
      }
      else {
        $topics = $this->topicsForEntity($entity);
      }
      if ($this->isKeyEnabled(self::SETTING_CREATE_EVENT, $topics, $entity->getEntityTypeId(), $entity->bundle())) {
        return $this->buildContentForm($form, $form_state, $entity, $topics);
      }
    }
    else if ($route_name = $this->routeMatch->getRouteName()) {
      foreach ($this->entityTypeManager->getDefinitions() as $entity_type => $definition) {
        if (($definition instanceof ContentEntityTypeInterface) && $this->assertContentTypeSettingsForm($route_name, $definition)) {
          return $this->buildContentTypeSettingsForm($form, $form_state, $definition);
        }
      }
    }
    return FALSE;
  }

  /**
   * @param string $route_name
   * @param \Drupal\Core\Entity\ContentEntityTypeInterface $definition
   *
   * @return bool
   */
  private function assertContentTypeSettingsForm(string $route_name, ContentEntityTypeInterface $definition): bool {
    switch ($route_name) {
      case 'entity.taxonomy_vocabulary.overview_form':
        return FALSE;

      case 'entity.taxonomy_vocabulary.edit_form':
        return $definition->id() === 'taxonomy_term';

      default:
        if ($field_ui_base_route = $definition->get('field_ui_base_route')) {
          return $field_ui_base_route === $route_name;
        }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(EventInterface $event): Url {
    /** @var \Drupal\danse_content\Payload $payload */
    $payload = $event->getPayload();
    try {
      return $payload->getEntity()->toUrl();
    }
    catch (EntityMalformedException $e) {
      return Url::fromRoute('<front>');
    }
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return bool
   */
  public function isPublished(ContentEntityInterface $entity): bool {
    if ($entity->isNew()) {
      return FALSE;
    }
    $entityType = $entity->getEntityType();
    if ($entityType->hasKey('status')) {
      $key = $entityType->getKey('status');
    }
    elseif ($entityType->hasKey('published')) {
      $key = $entityType->getKey('published');
    }
    else {
      return TRUE;
    }
    return (bool) $entity->get($key)->value;
  }

  /**
   * @param string $key
   * @param \Drupal\danse_content\Topic\TopicInterface[] $topics
   * @param string $entity_type_id
   * @param string|null $bundle
   *
   * @return bool
   */
  protected function isKeyEnabled(string $key, array $topics, string $entity_type_id, string $bundle = NULL): bool {
    $config = $this->configFactory->get($this->configId($entity_type_id, $bundle));
    foreach ($topics as $topic) {
      if ($config->get($topic->getSettingKey($key))) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param \Drupal\Core\Entity\EntityTypeInterface $definition
   *
   * @return array
   */
  protected function buildContentTypeSettingsForm(array &$form, FormStateInterface $form_state, EntityTypeInterface $definition): array {
    if (($bundleKey = $definition->getBundleEntityType()) && $parameter = $this->routeMatch->getParameter($bundleKey)) {
      $bundle = $parameter->id();
    }
    else {
      $bundle = NULL;
    }
    $config_id = $this->configId($definition->id(), $bundle);
    $config = $this->configFactory->get($config_id);
    $form['danse_config_id'] = ['#type' => 'hidden', '#value' => $config_id];
    $roles = $this->query->rolesAsSelectList();

    $header = [''];
    foreach (self::topics() as $topic) {
      if ($topic->assert($definition->id())) {
        $header[] = $topic->settingsFormLabel();
      }
    }

    $form['danse'] = [
      '#type' => 'details',
      '#group' => isset($form['additional_settings']) ? 'additional_settings' : 'advanced',
      '#title' => $this->t('DANSE'),
      '#tree' => TRUE,
      '#weight' => 11,
    ];
    $form['danse']['events'] = [
      '#type' => 'table',
      '#sticky' => TRUE,
      '#header' => $header,
    ];
    foreach ($this->settings() as $setting => $settingDef) {
      $form['danse']['events'][$setting]['label'] = [
        '#markup' => $settingDef['label'],
      ];
      foreach ($this->defaultSettingLabels() as $key => $label) {
        $form['danse']['events'][$key]['label'] = [
          '#markup' => $label,
        ];
      }
      foreach (self::topics() as $topic) {
        if ($topic->assert($definition->id())) {
          $form['danse']['events'][$setting][$topic->id()] = [
            '#type' => 'checkbox',
            '#default_value' => $config->get($topic->getSettingKey($setting)),
          ];
          if ($topic->accessToDefaultSettings()) {
            foreach ($this->defaultSettings($roles, $config, $topic, FALSE) as $key => $element) {
              $form['danse']['events'][$key][$topic->id()] = $element;
              unset($form['danse']['events'][$key][$topic->id()]['#description']);
            }
          }
        }
      }
    }
    $form['danse']['events']['allow_subscription']['label'] = [
      '#markup' => $this->t('Allow entity type subscription'),
    ];
    $form['danse']['events']['allow_entity_subscription']['label'] = [
      '#markup' => $this->t('Allow individual entity subscription'),
    ];
    $form['danse']['events']['allow_related_entity_subscription']['label'] = [
      '#markup' => $this->t('Allow related entity subscription'),
    ];
    foreach (self::topics() as $topic) {
      if ($topic->assert($definition->id())) {
        $form['danse']['events']['allow_subscription'][$topic->id()] = [
          '#type' => 'select',
          '#options' => $roles,
          '#multiple' => TRUE,
          '#default_value' => $config->get($topic->getSettingKey('allow_subscription')),
        ];
        $form['danse']['events']['allow_entity_subscription'][$topic->id()] = [
          '#type' => 'select',
          '#options' => $roles,
          '#multiple' => TRUE,
          '#default_value' => $config->get($topic->getSettingKey('allow_entity_subscription')),
        ];
        $form['danse']['events']['allow_related_entity_subscription'][$topic->id()] = [
          '#type' => 'select',
          '#options' => $roles,
          '#multiple' => TRUE,
          '#default_value' => $config->get($topic->getSettingKey('allow_related_entity_subscription')),
        ];
      }
    }

    return [$this, 'submitContentTypeSettingsForm'];
  }

  /**
   * Submit callback.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitContentTypeSettingsForm(array &$form, FormStateInterface $form_state): void {
    $config_id = $form_state->getValue(['danse_config_id']);
    $config = $this->configFactory->getEditable($config_id);
    $config
      ->set('events', $form_state->getValue(['danse', 'events']))
      ->save();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param \Drupal\danse_content\Topic\TopicInterface[] $topics
   *
   * @return array
   */
  protected function buildContentForm(array &$form, FormStateInterface $form_state, ContentEntityInterface $entity, array $topics): array {
    $config_id = $this->configId($entity->getEntityTypeId(), $entity->bundle());
    $config = $this->configFactory->get($config_id);
    $topicIds = [];
    foreach ($topics as $topic) {
      $topicIds[] = $topic->id();
    }
    $form['danse_topics'] = ['#type' => 'hidden', '#value' => implode(',', $topicIds)];
    $form['danse_was_published'] = ['#type' => 'hidden', '#value' => $this->isPublished($entity)];

    $accessSettings = FALSE;
    $userRoles = $this->currentUser->getRoles();
    foreach ($topics as $topic) {
      $relevantRoles = array_intersect(
        $config->get($topic->getSettingKey('access settings')) ?? [],
        $userRoles
      );
      if (!empty($relevantRoles)) {
        $accessSettings = TRUE;
        break;
      }
    }

    $form['danse'] = [
      '#type' => 'details',
      '#group' => isset($form['additional_settings']) ? 'additional_settings' : 'advanced',
      '#title' => $this->t('DANSE'),
      '#weight' => 11,
      '#access' => $accessSettings,
      '#tree' => TRUE,
    ];
    /** @noinspection AdditionOperationOnArraysInspection */
    $form['danse'] += $this->defaultSettings([], $config, $topics[0], TRUE);

    $entity->danse_processed_by_ui = TRUE;
    return [$this, 'submitContentForm'];
  }

  /**
   * Submit callback.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitContentForm(array &$form, FormStateInterface $form_state): void {
    $info = $form_state->getBuildInfo();
    /** @var \Drupal\Core\Entity\ContentEntityFormInterface $callback */
    $callback = $info['callback_object'];
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $callback->getEntity();
    $topics = [];
    foreach (explode(',', $form_state->getValue('danse_topics')) as $topicId) {
      $topics[] = self::topic($topicId);
    }
    $orgStatus = (bool) $form_state->getValue('danse_was_published');
    $values = $form_state->getValue('danse');
    $this->processEntity($entity, $topics, $orgStatus, $values);
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param \Drupal\danse_content\Topic\TopicInterface[] $topics
   * @param bool $orgStatus
   * @param array|null $values
   */
  public function processEntity(ContentEntityInterface $entity, array $topics, bool $orgStatus, array $values = NULL): void {
    static $processedEntities = [];
    if (!empty($entity->danse_processed_by_ui) && $values === NULL) {
      // We got here from an entity hook, but don't want to process that entity
      // right here, because it got edited through the UI and we want to grab
      // the settings from the entity edit form. So let's skip this, we will
      // be getting here a second time for this entity and process it then.
      return;
    }
    if (!empty($processedEntities[$entity->getEntityTypeId()][$entity->id()])) {
      // Avoid double processing.
      return;
    }
    $processedEntities[$entity->getEntityTypeId()][$entity->id()] = TRUE;
    if ($values === NULL) {
      if (!$this->isKeyEnabled(self::SETTING_CREATE_EVENT, $topics, $entity->getEntityTypeId(), $entity->bundle())) {
        return;
      }
      $config_id = $this->configId($entity->getEntityTypeId(), $entity->bundle());
      $config = $this->configFactory->get($config_id);
      $values = [];
      foreach ($this->defaultSettings([], $config, $topics[0], FALSE) as $key => $value) {
        $values[$key] = $value['#default_value'];
      }
    }
    switch ($topics[0]->id()) {
      case TopicInterface::CREATE:
        $create = $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::CREATE)], $entity->getEntityTypeId(), $entity->bundle());
        $publish = $this->isPublished($entity) && $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::PUBLISH)], $entity->getEntityTypeId(), $entity->bundle());
        $this->createContent($entity, $values['push'], $values['force'], $values['silent'], $create, $publish);
        break;

      case TopicInterface::UPDATE:
        $update = $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::UPDATE)], $entity->getEntityTypeId(), $entity->bundle());
        $newStatus = $this->isPublished($entity);
        $publish = FALSE;
        $unpublish = FALSE;
        if ($newStatus !== $orgStatus) {
          if ($newStatus) {
            $publish = $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::PUBLISH)], $entity->getEntityTypeId(), $entity->bundle());
          }
          else {
            $unpublish = $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::UNPUBLISH)], $entity->getEntityTypeId(), $entity->bundle());
          }
        }
        $this->updateContent($entity, $values['push'], $values['force'], $values['silent'], $update, $publish, $unpublish);
        break;

      case TopicInterface::DELETE:
        $delete = $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::DELETE)], $entity->getEntityTypeId(), $entity->bundle());
        $unpublish = $this->isPublished($entity) && $this->isKeyEnabled(self::SETTING_CREATE_EVENT, [self::topic(TopicInterface::UNPUBLISH)], $entity->getEntityTypeId(), $entity->bundle());
        $this->deleteContent($entity, $values['push'], $values['force'], $values['silent'], $delete, $unpublish);
        break;
    }
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   * @param bool $create
   * @param bool $publish
   */
  protected function createContent(ContentEntityInterface $entity, bool $push, bool $force, bool $silent, bool $create, bool $publish): void {
    if ($create) {
      // Make create event silent if we also publish the entity afterwards.
      $createSlient = $publish ? TRUE : $silent;
      $this->createEvent(TopicInterface::CREATE, $entity->label(), new Payload($entity), $push, $force, $createSlient);
    }
    if ($publish) {
      $this->publishContent($entity, $push, $force, $silent);
    }
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   * @param bool $update
   * @param bool $publish
   * @param bool $unpublish
   */
  protected function updateContent(ContentEntityInterface $entity, bool $push, bool $force, bool $silent, bool $update, bool $publish, bool $unpublish): void {
    if ($update) {
      $this->createEvent(TopicInterface::UPDATE, $entity->label(), new Payload($entity), $push, $force, $silent);
      // Make sure that a potential second event won't create duplicate
      // notifications.
      $silent = TRUE;
    }
    if ($publish) {
      $this->publishContent($entity, $push, $force, $silent);
    }
    if ($unpublish) {
      $this->unpublishContent($entity, $push, $force, $silent);
    }
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   * @param bool $delete
   * @param bool $unpublish
   */
  protected function deleteContent(ContentEntityInterface $entity, bool $push, bool $force, bool $silent, bool $delete, bool $unpublish): void {
    if ($unpublish) {
      $this->unpublishContent($entity, $push, $force, $silent);
      // Make sure that a potential second event won't create duplicate
      // notifications.
      $silent = TRUE;
    }
    if ($delete) {
      $this->createEvent(TopicInterface::DELETE, $entity->label(), new Payload($entity), $push, $force, $silent);
    }
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   */
  protected function publishContent(ContentEntityInterface $entity, bool $push, bool $force, bool $silent): void {
    $this->createEvent(TopicInterface::PUBLISH, $entity->label(), new Payload($entity), $push, $force, $silent);
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   */
  protected function unpublishContent(ContentEntityInterface $entity, bool $push, bool $force, bool $silent): void {
    $this->createEvent(TopicInterface::UNPUBLISH, $entity->label(), new Payload($entity), $push, $force, $silent);
  }

}
