<?php

namespace Drupal\danse_content\Plugin\views\filter;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\views\Plugin\views\filter\InOperator;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a filter for subscribed entities.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("danse_subscription")
 */
class Subscription extends InOperator {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * Creates an instance of ModerationStateFilter.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Session\AccountInterface $current_user
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, Connection $database, AccountInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('database'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    return Cache::mergeTags(parent::getCacheTags(), $this->entityTypeManager->getDefinition('user')->getListCacheTags());
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    return Cache::mergeContexts(parent::getCacheContexts(), $this->entityTypeManager->getDefinition('user')->getListCacheContexts());
  }

  /**
   * {@inheritdoc}
   */
  public function canExpose(): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    // Nothing to do
  }

  /**
   * {@inheritdoc}
   */
  protected function opSimple(): void {
    try {
      $entity_type = $this->entityTypeManager->getDefinition($this->getEntityType());
    }
    catch (PluginNotFoundException | Exception $e) {
      $entity_type = NULL;
      // TODO: Log this exception.
    }
    if ($entity_type) {
      $this->ensureMyTable();
      $entity_base_table = $entity_type->getBaseTable();
      $field = implode('.', [
        $entity_base_table,
        $entity_type->getKey('id'),
      ]);
      $query = $this->database->select('users_data', 'u')
        ->fields('u', ['name'])
        ->condition('u.uid', $this->currentUser->id())
        ->condition('u.module', 'danse')
        ->condition('u.value', 1)
        ->condition('u.name', implode('-', [
          'content',
          $entity_type->id(),
          '%',
        ]), 'LIKE')
        ->execute();
      $values = [];
      if ($query) {
        foreach ($query->fetchCol() as $item) {
          if ($value = (int) explode('-', $item)[2]) {
            $values[] = $value;
          }
        }
      }
      if (empty($values)) {
        // Make sure we get an empty result for those without any subscription.
        $values = [0];
      }
      $this->query->addWhere($this->options['group'], $field, $values, 'IN');
    }
  }

}
