<?php

namespace Drupal\danse;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Plugin\PluginBase as CorePluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Site\Settings;
use Drupal\danse\Entity\Event;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\Entity\Notification;
use Drupal\danse\Entity\NotificationAction;
use Drupal\push_framework\ChannelPluginInterface;
use Drupal\push_framework\Plugin\DanseRecipientSelection\DirectPushInterface;
use Drupal\user\UserDataInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for DANSE plugins.
 */
abstract class PluginBase extends CorePluginBase implements PluginInterface, ContainerFactoryPluginInterface {

  use DependencySerializationTrait;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $bundleInfo;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\user\UserDataInterface
   */
  protected $userData;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * @var \Drupal\danse\RecipientSelectionInterface[]
   */
  protected $recipientSelectionPlugins;

  /**
   * @var \Drupal\danse\Query
   */
  protected $query;

  /**
   * @var array
   */
  protected $subscriptionKeyParts;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * PluginBase constructor.
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_info
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\user\UserDataInterface $user_data
   * @param \Drupal\danse\RecipientSelectionManager $recipient_selection_manager
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   * @param \Drupal\danse\Query $query
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ModuleHandlerInterface $module_handler, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $bundle_info, ConfigFactoryInterface $config_factory, UserDataInterface $user_data, RecipientSelectionManager $recipient_selection_manager, AccountProxyInterface $current_user, RouteMatchInterface $route_match, Query $query) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
    $this->bundleInfo = $bundle_info;
    $this->configFactory = $config_factory;
    $this->userData = $user_data;
    $this->currentUser = $current_user;
    $this->routeMatch = $route_match;
    $this->query = $query;
    $this->recipientSelectionPlugins = [];

    $ids = $config_factory->get('danse.settings')->get('recipient_selection_plugin.' . $this->getPluginId());
    if (!empty($ids)) {
      foreach (explode(',', $ids) as $danse_plugin_id) {
        try {
          $this->recipientSelectionPlugins[] = $recipient_selection_manager->createInstance($danse_plugin_id);
        } catch (PluginException $e) {
          // This may fail, i.e. during enable/disable module.
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('config.factory'),
      $container->get('user.data'),
      $container->get('plugin.manager.danse.recipient.selection'),
      $container->get('current_user'),
      $container->get('current_route_match'),
      $container->get('danse.query')
    );
  }

  /**
   * {@inheritdoc}
   */
  final public function label(): string {
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  final public function createNotifications(): void {
    /** @var \Drupal\danse\Entity\EventInterface $event */
    foreach ($this->getUnprocessedEvents() as $event) {
      /** @var \Drupal\danse\PayloadInterface $payload */
      if ($payload = $event->getPayload()) {
        $subscribers = $this->getSubscribers($event);
        $recipients = [];
        if ($event->isPush()) {
          foreach ($this->recipientSelectionPlugins as $recipientSelectionPlugin) {
            if ($recipientSelectionPlugin instanceof DirectPushInterface) {
              $channelPluginId = $recipientSelectionPlugin->directPushChannelId();
              if (Settings::get('danse_notification_delivery', TRUE)) {
                $result = $recipientSelectionPlugin->push($payload);
              }
              else {
                $result = ChannelPluginInterface::RESULT_STATUS_SUCCESS;
              }
              $success = $result === ChannelPluginInterface::RESULT_STATUS_SUCCESS;
              foreach ($this->createNotification($event, 'push', 1) as $notification) {
                if ($success) {
                  $notification->markDelivered();
                }
                $action = NotificationAction::create([
                  'notification' => $notification->id(),
                  'success' => $success,
                  'payload' => [
                    'channel plugin' => $channelPluginId,
                  ],
                ]);
                try {
                  $action->save();
                }
                catch (EntityStorageException $e) {
                  // @todo Log this exception.
                }
              }
            }
            else {
              foreach ($recipientSelectionPlugin->getRecipients($payload) as $uid) {
                if (!in_array($uid, $recipients, TRUE)) {
                  $recipients[] = $uid;
                }
              }
            }
          }
        }
        foreach ($subscribers as $subscriber) {
          $this->createNotification($event, 'subscription', $subscriber);
        }
        foreach ($recipients as $recipient) {
          if (!in_array($recipient, $subscribers, TRUE)) {
            $this->createNotification($event, 'push', $recipient);
          }
        }
      }
      $event->setProcessed();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function assertSubscriptionKey() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  final public function setSubscriptionKeyParts(array $parts): void {
    $this->subscriptionKeyParts = $parts;
  }

  /**
   * {@inheritdoc}
   */
  final public function subscriptionKey(string ...$parts): string {
    array_unshift($parts, $this->getPluginId());
    return implode('-', $parts);
  }

  /**
   * @param string $topic_id
   * @param string $label
   * @param \Drupal\danse\PayloadInterface $payload
   * @param bool $push
   * @param bool $force
   * @param bool $silent
   *
   * @return \Drupal\danse\Entity\EventInterface|null
   */
  protected function createEvent($topic_id, $label, PayloadInterface $payload, $push = TRUE, $force = FALSE, $silent = FALSE): ?EventInterface {
    if (!$this->assertPayload($payload)) {
      // TODO: Log "Invalid payload provided.".
      return NULL;
    }
    /** @var EventInterface $event */
    $event = Event::create([
      'plugin' => $this->getPluginId(),
      'topic' => $topic_id,
      'label' => $label,
      'payload' => $payload,
      'push' => $push,
      'force' => $force,
      'silent' => $silent,
    ]);
    try {
      $event->save();
    } catch (EntityStorageException $e) {
      // TODO: Log the issue
      return NULL;
    }
    return $event;
  }

  /**
   *
   * @param \Drupal\danse\Entity\EventInterface $event
   *
   * @return int[]
   */
  protected function getSubscribers(EventInterface $event): array {
    $references = $event->getPayload()->getSubscriptionReferences($event);
    $uids = [];
    foreach ($references as $reference) {
      foreach ($this->userData->get('danse', NULL, $reference) as $uid => $flag) {
        if ($flag) {
          $uids[] = $uid;
        }
      }
    }
    return array_unique($uids, SORT_NUMERIC);
  }

  /**
   * @param \Drupal\danse\Entity\EventInterface $event
   * @param string $trigger
   * @param int $uid
   *
   * @return \Drupal\danse\Entity\NotificationInterface[]
   */
  protected function createNotification(EventInterface $event, $trigger, $uid): array {
    if (!$event->getPayload()->hasAccess($uid)) {
      return [];
    }
    $existingNotifications = $this->query->findSimilarEventNotifications($event, $uid);
    if (!empty($existingNotifications) && !$event->isForce()) {
      // There is an existing notification which is undelivered, so we do not create a new one.
      return $existingNotifications;
    }
    /** @var \Drupal\danse\Entity\NotificationInterface $notification */
    $notification = Notification::create([
      'event' => $event,
      'trigger' => $trigger,
      'uid' => $uid,
    ]);
    try {
      $notification->save();
    } catch (EntityStorageException $e) {
      // TODO: Log the issue
      return [];
    }
    // Mark existing notifications as redundant.
    foreach ($existingNotifications as $existingNotification) {
      $existingNotification->setSuccessor($notification);
    }
    return [$notification];
  }

  /**
   * @return \Drupal\danse\Entity\EventInterface[]
   */
  protected function getUnprocessedEvents(): array {
    try {
      /** @var \Drupal\danse\Entity\EventInterface[] $events */
      $events = $this->entityTypeManager->getStorage('danse_event')
        ->loadByProperties([
          'plugin' => $this->getPluginId(),
          'processed' => 0,
        ]);
      ksort($events);
      return $events;
    }
    catch (InvalidPluginDefinitionException $e) {
      // TODO: handle exception.
    }
    catch (PluginNotFoundException $e) {
      // TODO: handle exception.
    }
    return [];
  }

}
