<?php

namespace Drupal\danse;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\UserDataInterface;

/**
 * Logger service to write messages and exceptions to an external service.
 */
class Service {

  use StringTranslationTrait;

  /**
   * @var \Drupal\danse\PluginManager
   */
  protected PluginManager $pluginManager;

  /**
   * @var \Drupal\user\UserDataInterface
   */
  protected UserDataInterface $userData;

  /**
   * @var \Drupal\danse\Query
   */
  protected Query $query;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   * @param \Drupal\danse\PluginManager $plugin_manager
   * @param \Drupal\user\UserDataInterface $user_data
   * @param \Drupal\danse\Query $query
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(ConfigFactoryInterface $configFactory, PluginManager $plugin_manager, UserDataInterface $user_data, Query $query, EntityTypeManagerInterface $entity_type_manager) {
    $this->pluginManager = $plugin_manager;
    $this->userData = $user_data;
    $this->query = $query;
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $configFactory->get('danse.settings');
  }

  /**
   * @return \Drupal\danse\PluginInterface[]
   */
  public function getPluginInstances(): array {
    $instances = [];
    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      try {
        $instances[$id] = $this->pluginManager->createInstance($id);
      }
      catch (PluginException $e) {
        // We can savely ignore this here.
      }
    }
    return $instances;
  }

  /**
   * @param string $pluginId
   *
   * @return \Drupal\danse\PluginInterface|null
   */
  public function getPluginInstance(string $pluginId): ?PluginInterface {
    $plugin = NULL;
    try {
      /** @var \Drupal\danse\PluginBase $plugin */
      $plugin = $this->pluginManager->createInstance($pluginId);
    }
    catch (PluginException $e) {
      // TODO: Log this exception.
    }
    return $plugin;
  }

  /**
   * @param string $key
   *
   * @return \Drupal\danse\PluginInterface|null
   */
  public function getPluginInstanceFromSubscriptionKey(string $key): ?PluginInterface {
    $parts = explode('-', $key);
    try {
      $pluginId = array_shift($parts);
      /** @var \Drupal\danse\PluginBase $plugin */
      $plugin = $this->pluginManager->createInstance($pluginId);
      $plugin->setSubscriptionKeyParts($parts);
      return $plugin;
    }
    catch (PluginException $e) {
      // TODO: Log this exception.
    }
    return NULL;
  }

  /**
   * @param array $form
   * @param array|string $callback
   */
  protected function addSubmitHandler(array &$form, $callback): void {
    $callback = is_string($callback) ? [$this, $callback] : $callback;
    if (isset($form['actions']['submit']['#submit'])) {
      foreach (array_keys($form['actions']) as $action) {
        if ($action !== 'preview'
          && isset($form['actions'][$action]['#type'])
          && $form['actions'][$action]['#type'] === 'submit') {
          $form['actions'][$action]['#submit'][] = $callback;
        }
      }
    }
    else {
      $form['#submit'][] = $callback;
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function buildForm(array &$form, FormStateInterface $form_state): void {
    $submitHandler = [];
    foreach ($this->getPluginInstances() as $plugin) {
      if ($result = $plugin->buildForm($form, $form_state)) {
        $submitHandler[] = $result;
      }
    }
    foreach ($submitHandler as $item) {
      $this->addSubmitHandler($form, $item);
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param bool $user_edit_mode
   */
  public function buildUserSubscriptionForm(array &$form, AccountInterface $account, bool $user_edit_mode = TRUE): void {
    $checkboxes = [];
    if ((bool) $this->config->get('subscriptions_as_tab.events') !== $user_edit_mode) {
      $roles = $account->getRoles();
      foreach ($this->getPluginInstances() as $plugin) {
        foreach ($plugin->getSupportedSubscriptions($roles) as $key => $title) {
          $checkboxes[$key] = [
            '#type' => 'checkbox',
            '#title' => $title,
            '#default_value' => $this->userData->get('danse', $account->id(), $key),
          ];
        }
      }
    }

    $views = [];
    if ((bool) $this->config->get('subscriptions_as_tab.view') !== $user_edit_mode) {
      foreach ($this->entityTypeManager->getStorage('view')->loadByProperties([
          'status' => 1,
          'tag' => 'DANSE Subscription',
        ]) as $view) {
        $views[] = [views_embed_view($view->id())];
      }
    }

    if (!empty($checkboxes) || !empty($views)) {
      if ($user_edit_mode) {
        $form['danse_subscriptions'] = [
          '#type' => 'details',
          '#title' => $this->t('Subscriptions'),
          '#open' => FALSE,
          '#tree' => TRUE,
          '#weight' => 9,
        ];
      }
      else {
        $form['danse_subscriptions'] = [
          '#type' => 'container',
          '#tree' => TRUE,
        ];
      }

      if (!empty($views)) {
        $form['danse_subscriptions']['view'] = $views;
      }

      if (!empty($checkboxes)) {
        if (empty($views)) {
          $form['danse_subscriptions']['settings'] = [
            '#type' => 'container',
            '#tree' => TRUE,
          ];
        }
        else {
          $form['danse_subscriptions']['settings'] = [
            '#type' => 'details',
            '#title' => $this->t('Settings'),
            '#open' => FALSE,
            '#tree' => TRUE,
          ];
        }
        /** @noinspection AdditionOperationOnArraysInspection */
        $form['danse_subscriptions']['settings'] += $checkboxes;
        if (!$user_edit_mode) {
          $form['danse_subscriptions']['settings']['actions'] = ['#type' => 'actions'];
          $form['danse_subscriptions']['settings']['actions']['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Save subscriptions'),
            '#button_type' => 'primary',
          ];
        }
        $form['danse_account'] = ['#type' => 'value', '#value' => $account];
        $this->addSubmitHandler($form, 'submitUserSubscriptionForm');
      }
    }
  }

  /**
   * Submit callback.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitUserSubscriptionForm(array &$form, FormStateInterface $form_state): void {
    $this->saveUserSubscriptionForm($form_state, $form_state->getValue('danse_account'));
  }

  /**
   * Save user settings.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param \Drupal\Core\Session\AccountInterface $account
   */
  public function saveUserSubscriptionForm(FormStateInterface $form_state, AccountInterface $account): void {
    foreach ($form_state->getValue(['danse_subscriptions', 'settings']) as $key => $value) {
      $this->userData->set('danse', $account->id(), $key, (int) $value);
    }
  }

  /**
   * Create all outstanding notifications.
   */
  public function createNotifications(): void {
    foreach ($this->getPluginInstances() as $plugin) {
      $plugin->createNotifications();
    }
  }

  /**
   * @param \Drupal\danse\PayloadInterface $payload
   */
  public function markSeen(PayloadInterface $payload): void {
    foreach ($this->query->findNotificationsForCurrentUser($payload) as $notification) {
      $notification->markSeen();
    }
  }

}
